@@ -37,6 +37,8 @@ pstring username="";
 BOOL got_pass = False;
 BOOL connect_as_printer = False;
 
+extern struct in_addr myip;
+
 pstring debugf = DEBUGFILE;
 
 
@@ -116,7 +118,7 @@ void show_finfo(file_info *finfo)
 /****************************************************************************
 check for existance of a dir
 ****************************************************************************/
-BOOL chkpath(char *path)
+BOOL chkpath(char *path,BOOL report)
 {
   pstring inbuf,outbuf;
   char *p;
@@ -164,11 +166,8 @@ void cmd_cd(char *inbuf,char *outbuf )
       clean_name(cur_dir);
 
       if (!strequal(cur_dir,"\\"))
-	if (!chkpath(dname))
-	  {
-	    Debug(0,"cd %s : %s\n", cur_dir,smb_errstr(inbuf));
-	    strcpy(cur_dir,saved_dir);
-	  }
+	if (!chkpath(dname,True))
+	  strcpy(cur_dir,saved_dir);
     }
   else
     Debug(0,"Current directory is %s\n",cur_dir);
@@ -179,7 +178,7 @@ void cmd_cd(char *inbuf,char *outbuf )
 /****************************************************************************
 do a directory listing, calling fn on each file found
 ****************************************************************************/
-void do_dir(char *inbuf,char *outbuf,char *mask,int attribute,void (*fn)())
+void do_dir(char *inbuf,char *outbuf,char *Mask,int attribute,void (*fn)(),BOOL recurse_dir)
 {
   char *p;
   int received = 0;
@@ -188,6 +187,10 @@ void do_dir(char *inbuf,char *outbuf,char *mask,int attribute,void (*fn)())
   int num_asked = (max_xmit - 100)/DIR_STRUCT_SIZE;
   int num_received = 0;
   char *dirlist = NULL;
+  pstring mask;
+
+  strcpy(mask,Mask);
+  expand_mask(mask);
   
   while (1)
     {
@@ -231,6 +234,8 @@ void do_dir(char *inbuf,char *outbuf,char *mask,int attribute,void (*fn)())
 
       Debug(5,"dir received %d\n",received);
 
+      Debug(6,"errstr=%s\n",smb_errstr(inbuf));
+
       if (received <= 0) break;
 
       dirlist = Realloc(dirlist,(num_received + received)*DIR_STRUCT_SIZE);
@@ -247,6 +252,8 @@ void do_dir(char *inbuf,char *outbuf,char *mask,int attribute,void (*fn)())
       memcpy(status,p + ((received-1)*DIR_STRUCT_SIZE),21);
 
       num_received += received;
+
+      if (CVAL(inbuf,smb_rcls) != 0) break;
     }
 
   received = num_received;
@@ -273,18 +280,37 @@ void do_dir(char *inbuf,char *outbuf,char *mask,int attribute,void (*fn)())
       if (finfo.mode & aHIDDEN) strcat(attrstr,"H");
       if (finfo.mode & aSYSTEM) strcat(attrstr,"S");
       if (finfo.mode & aRONLY) strcat(attrstr,"R");	  
+      if (!sane_unix_date(finfo.mtime))
+ 	strcat(attrstr,"I");
       
       if (!((finfo.mode & aDIR) == 0 && *fileselection && 
-	    !mask_match(finfo.name,fileselection,False,False)))
-	{
-	  if (fn)
-	    fn(&finfo);
-	  else
-	    Debug(0,"%20.20s%7.7s%10d  %s",
-		  finfo.name,
-		  attrstr,
-		  finfo.size,
-		  asctime(LocalTime(&finfo.mtime)));
+ 	    !mask_match(finfo.name,fileselection,False,False)) &&
+ 	  !(recurse_dir && (strequal(finfo.name,".") || 
+ 			    strequal(finfo.name,".."))))
+  	{
+ 	  if (recurse_dir && (finfo.mode & aDIR))
+ 	    {
+ 	      pstring mask2;
+ 	      pstring sav_dir;
+ 	      strcpy(sav_dir,cur_dir);
+ 	      strcat(cur_dir,finfo.name);
+ 	      strcat(cur_dir,"\\");
+ 	      strcpy(mask2,cur_dir);
+ 	      strcat(mask2,"*.*");
+ 	      do_dir(inbuf,outbuf,mask2,attribute,fn,True);
+ 	      strcpy(cur_dir,sav_dir);
+ 	    }
+  	  else
+ 	    {
+ 	      if (fn)
+ 		fn(&finfo);
+ 	      else
+		Debug(0,"%20.20s%7.7s%10d  %s",
+ 		      finfo.name,
+ 		      attrstr,
+ 		      finfo.size,
+ 		      asctime(LocalTime(&finfo.mtime)));
+ 	    }
 	}
       p += DIR_STRUCT_SIZE;
     }
@@ -316,7 +342,7 @@ void cmd_dir(char *inbuf,char *outbuf)
   else
     strcat(mask,"*.*");
 
-  do_dir(inbuf,outbuf,mask,attribute,NULL);
+  do_dir(inbuf,outbuf,mask,attribute,NULL,False);
 }
 
 
@@ -835,7 +861,7 @@ void do_mget(file_info *finfo)
       strcat(mget_mask,"*.*");
       
       do_dir((char *)inbuf,(char *)outbuf,
-	     mget_mask,aSYSTEM | aHIDDEN | aDIR,do_mget);
+	     mget_mask,aSYSTEM | aHIDDEN | aDIR,do_mget,False);
 	chdir("..");
       strcpy(cur_dir,saved_curdir);
       free(inbuf);free(outbuf);
@@ -883,7 +909,7 @@ void cmd_mget(char *inbuf,char *outbuf)
 	strcat(mget_mask,p);
     }
 
-  do_dir((char *)inbuf,(char *)outbuf,mget_mask,attribute,do_mget);
+  do_dir((char *)inbuf,(char *)outbuf,mget_mask,attribute,do_mget,False);
 }
 
 /****************************************************************************
@@ -1015,7 +1041,7 @@ void cmd_mkdir(char *inbuf,char *outbuf)
       while (p)
 	{
 	  strcat(ddir2,p);
-	  if (!chkpath(ddir2))
+	  if (!chkpath(ddir2,False))
 	    {		  
 	      do_mkdir(ddir2);
 	    }
@@ -1186,6 +1212,134 @@ void do_put(char *rname,char *lname,file_info *finfo)
   free(inbuf);free(outbuf);
 }
 
+ 
+/****************************************************************************
+  fudge a single file
+****************************************************************************/
+void do_fudge(file_info *finfo)
+{
+  int fnum;
+  char *p;
+  char *inbuf,*outbuf;
+  time_t old_mtime;
+  static time_t sane_date = 0;
+  
+  {
+    pstring n2;
+    strcpy(n2,finfo->name);
+    strcpy(finfo->name,cur_dir);
+    strcat(finfo->name,n2);
+  }
+  
+  if (sane_date == 0)
+    {
+      sane_date = start_of_month();
+    }
+  
+  if (sane_unix_date(finfo->mtime))
+    return;
+  
+  Debug(2,"Fixing insane date on file %s %s",finfo->name,
+ 	asctime(LocalTime(&finfo->mtime)));
+  
+  finfo->mtime = sane_date;
+  
+  inbuf = (char *)malloc(BUFFER_SIZE);
+  outbuf = (char *)malloc(BUFFER_SIZE);
+  
+  if (!inbuf || !outbuf)
+    {
+      Debug(0,"out of memory\n");
+      return;
+    }
+  
+  memset(outbuf,0,smb_size);
+  set_message(outbuf,2,2 + strlen(finfo->name));
+  
+  CVAL(outbuf,smb_com) = SMBopen;
+  SSVAL(outbuf,smb_tid,cnum);
+  setup_pkt(outbuf);
+  
+  SSVAL(outbuf,smb_vwv0,0);
+  SSVAL(outbuf,smb_vwv1,0);
+  
+  p = smb_buf(outbuf);
+  *p++ = 4;      
+  strcpy(p,finfo->name);
+  clean_name(finfo->name);
+  
+  send_smb(outbuf);
+  receive_smb(inbuf,0);
+  
+  if (CVAL(inbuf,smb_rcls) != 0)
+    {
+      Debug(0,"Error %s opening remote file %s\n",smb_errstr(inbuf),
+ 	    finfo->name);
+      free(inbuf);free(outbuf);
+      return;
+    }
+  
+  fnum = SVAL(inbuf,smb_vwv0);
+  old_mtime = IVAL(inbuf,smb_vwv2);
+  
+  if (sane_unix_date(old_mtime))
+    {
+      finfo->mtime = old_mtime;
+      Debug(2,"Already sane date on file %s %s",finfo->name,
+ 	    asctime(LocalTime(&finfo->mtime)));
+    }
+  
+  memset(outbuf,0,smb_size);
+  set_message(outbuf,3,0);
+  CVAL(outbuf,smb_com) = SMBclose;
+  SSVAL(outbuf,smb_tid,cnum);
+  setup_pkt(outbuf);
+  
+  SSVAL(outbuf,smb_vwv0,fnum);
+  SIVAL(outbuf,smb_vwv1,finfo->mtime);
+  
+  send_smb(outbuf);
+  receive_smb(inbuf,0);
+  
+  if (CVAL(inbuf,smb_rcls) != 0)
+    {
+      Debug(0,"Error %s closing remote file\n",smb_errstr(inbuf));
+      free(inbuf);free(outbuf);
+      return;
+    }
+  
+  free(inbuf);free(outbuf);
+}
+
+/****************************************************************************
+  fudge dates
+  ****************************************************************************/
+void cmd_fudge(char *inbuf,char *outbuf)
+{
+  int attribute = aDIR | aSYSTEM | aHIDDEN;
+  pstring mask;
+  char *p;
+  
+  strcpy(mask,cur_dir);
+  if(mask[strlen(mask)-1]!='\\')
+    strcat(mask,"\\");
+  
+  p = strtok(NULL,SEPARATORS);
+  if (p)
+    {
+      if (*p == '\\')
+ 	strcpy(mask,p);
+      else
+ 	strcat(mask,p);
+    }
+  else
+    strcat(mask,"*.*");
+  
+  do_dir(inbuf,outbuf,mask,attribute,do_fudge,recurse);
+}
+
+ 
+
 /****************************************************************************
 put a file
 ****************************************************************************/
@@ -1253,7 +1407,6 @@ void cmd_select(void)
 }
 
 
-
 /****************************************************************************
 mput some files
 ****************************************************************************/
@@ -1533,6 +1686,90 @@ void do_del(file_info *finfo)
 }
 
 
+/****************************************************************************
+try and browse available connections on a host
+****************************************************************************/
+void browse_host(char *name)
+{
+#if 1
+  char *inbuf,*outbuf;
+  char *p;
+
+  inbuf = (char *)malloc(BUFFER_SIZE);
+  outbuf = (char *)malloc(BUFFER_SIZE);
+
+  memset(outbuf,0,177);
+
+  strupper(name);
+
+  if (!have_ip)
+    {
+      struct hostent *hp;
+
+      if ((hp = gethostbyname(name)) == 0) 
+	{
+	  Debug(0,"Gethostbyname: Unknown host %s.\n",name);
+	  return;
+	}
+
+      memcpy(&dest_ip,hp->h_addr,4);
+    }
+
+  /* begin constructing the packet, I don't have a spec for this so
+     one example will have to do, with a lot of magic numbers */
+  CVAL(outbuf,0) = 021;
+  CVAL(outbuf,1) = 02;
+  CVAL(outbuf,2) = 00;
+  CVAL(outbuf,3) = 0112;
+  CVAL(outbuf,4) = 0226;
+  CVAL(outbuf,5) = 0313;
+  CVAL(outbuf,6) = 017;
+  CVAL(outbuf,7) = 0124;
+  CVAL(outbuf,8) = 00;
+  CVAL(outbuf,9) = 0212;
+  CVAL(outbuf,10) = 00;
+  CVAL(outbuf,11) = 0243;
+  CVAL(outbuf,12) = 00;
+  CVAL(outbuf,13) = 00;
+
+  p = outbuf + 14;
+  name_mangle(myname,p);
+  p += name_len(p);
+  name_mangle(name,p);
+  p += name_len(p);
+
+  strcat(p,"\377SMB%");
+
+  p = outbuf + 106;
+
+  strcat(p,"Ń");
+
+  CVAL(outbuf,117) = 07;
+  CVAL(outbuf,125) = 02;
+  CVAL(outbuf,135) = 0126;
+  CVAL(outbuf,137) = 07;
+  CVAL(outbuf,139) = 'X';
+  CVAL(outbuf,141) = 3;
+  CVAL(outbuf,143) = 1;
+  CVAL(outbuf,147) = 02;
+  CVAL(outbuf,149) = 032;
+
+  p = outbuf + 151;
+
+  strcat(p,"\\MAILSLOT\\BROWSE");
+
+  p += strlen(p) + 1;
+
+  strcat(p,"S 	");
+  CVAL(outbuf,176) = 1;
+
+  log_out(outbuf,177);  
+
+  send_packet(outbuf,177,&dest_ip,138,SOCK_DGRAM);
+#endif
+}
+
+
 
 /****************************************************************************
 delete some files
@@ -1556,7 +1793,7 @@ void cmd_del(char *inbuf,char *outbuf )
     }
   strcat(mask,p);
 
-  do_dir((char *)inbuf,(char *)outbuf,mask,attribute,do_del);
+  do_dir((char *)inbuf,(char *)outbuf,mask,attribute,do_del,False);
 }
 
 
@@ -1798,7 +2035,7 @@ BOOL send_login(char *inbuf,char *outbuf )
   memcpy(&servertime,inbuf+smb_vwv8,sizeof(servertime));
   servertime = make_unix_date(servertime);
 
-  Debug(3,"crypt key %s\n",smb_buf(inbuf));
+  Debug(3,"Got %d byte crypt key\n",strlen(smb_buf(inbuf)));
 
   Debug(1,"Server time is %s\n",asctime(LocalTime(&servertime)));
 
@@ -1820,7 +2057,7 @@ BOOL send_login(char *inbuf,char *outbuf )
       /* send a session setup command */
       memset(outbuf,0,smb_size);
       set_message(outbuf,10,2 + strlen(username) + strlen(pass));
-      CVAL(outbuf,smb_com) = SMBsesssetup;
+      CVAL(outbuf,smb_com) = SMBsesssetupX;
       setup_pkt(outbuf);
 
       CVAL(outbuf,smb_vwv0) = 0xFF;
@@ -1851,6 +2088,7 @@ BOOL send_login(char *inbuf,char *outbuf )
     strcpy(pass,"");
 #endif
 
+ again:
   set_message(outbuf,0,6 + strlen(service) + strlen(pass) + strlen(dev));
   CVAL(outbuf,smb_com) = SMBtcon;
   setup_pkt(outbuf);
@@ -1868,6 +2106,15 @@ BOOL send_login(char *inbuf,char *outbuf )
   send_smb(outbuf);
   receive_smb(inbuf,0);
 
+  /* trying again with a blank password */
+  if (CVAL(inbuf,smb_rcls) != 0 && 
+      strlen(pass) > 0 && 
+      Protocol >= PROT_LANMAN1)
+    {
+      strcpy(pass,"");
+      goto again;
+    }  
+
   if (CVAL(inbuf,smb_rcls) != 0)
     {
       Debug(0,"SMBtcon failed. %s\n",smb_errstr(inbuf));
@@ -1992,12 +2239,14 @@ BOOL open_sockets(int port )
   pstring service2;
   extern int Client;
 
+  strupper(service);
+
   strcpy(service2,service);
   host = strtok(service2,"\\/");
   strcpy(desthost,host);
   if (*myname == 0)
     {
-      gethostname(myname,sizeof(myname));
+      get_myname(myname,NULL);
       strupper(myname);
     }
 
@@ -2074,12 +2323,31 @@ void wait_keyboard(char *buffer)
       if (FD_ISSET(Client,&fds))
 	receive_smb(buffer,0);
 
-      chkpath("\\");
+      chkpath("\\",False);
     }
 
 }
 
 
+/****************************************************************************
+try and register my own netbios name with a unicast
+****************************************************************************/
+void register_myname(void)
+{
+  name_struct name;
+
+  name.valid = True;
+  strcpy(name.name,myname);
+  strupper(name.name);
+  strcpy(name.flags,"");
+  name.ip = myip;
+  name.ttl = 0;
+  name.nb_flags = 0;
+
+  register_name(&name,&dest_ip,NULL);
+}
+
+
 /****************************************************************************
   process commands from the client
 ****************************************************************************/
@@ -2093,6 +2361,9 @@ void process(void )
     return;
   
   memset(OutBuffer,0,smb_size);
+
+  register_myname();
+
   if (!send_login(InBuffer,OutBuffer))
     return;
 
@@ -2154,6 +2425,7 @@ void usage(char *pname)
 }
 
 
+
 /****************************************************************************
   main program
 ****************************************************************************/
@@ -2164,6 +2436,7 @@ int main(int argc,char *argv[])
   extern FILE *dbf;
   extern int DEBUGLEVEL;
   extern char *optarg;
+  pstring query_host="";
 
   DEBUGLEVEL = 2;
   dbf = stdout;
@@ -2210,9 +2483,12 @@ int main(int argc,char *argv[])
       argv++;
     }
 
-  while ((opt = getopt (argc, argv, "Nn:d:Pp:l:hI:EB:U:")) != EOF)
+  while ((opt = getopt (argc, argv, "Nn:d:Pp:l:hI:EB:U:Q:")) != EOF)
     switch (opt)
       {
+      case 'Q':
+	strcpy(query_host,optarg);
+	break;
       case 'U':
 	{
 	  char *p;
@@ -2283,7 +2559,15 @@ int main(int argc,char *argv[])
   /* Read the broadcast address from the interface */
   get_broadcast(&myip,&bcast_ip,&Netmask);
 #endif
+
+  get_myname(*myname?NULL:myname,&myip);  
   
+  if (*query_host)
+    {
+      browse_host(query_host);
+      return(0);
+    }
+
   if (open_sockets(port))
     {
       process();
