@@ -293,9 +293,7 @@ static parmmap aParmMaps[] =
    {"root",          E_ROOTDIR,      E_GLOBAL  },
    {"max xmit",      E_MAXXMIT,      E_GLOBAL  },
    {"keepalive",     E_KEEPALIVE,    E_GLOBAL  },
-   {"keep alive",    E_KEEPALIVE,    E_GLOBAL  },
    {"deadtime",      E_DEADTIME,     E_GLOBAL  },
-   {"dead time",     E_DEADTIME,     E_GLOBAL  },
    {"dont descend",  E_DONTDESCEND,  E_SERVICE  },
 };
 
