@@ -42,15 +42,6 @@ pstring myname="";
 pstring myhostname="";
 int myttl = 0;
 
-typedef struct
-{
-  BOOL valid;
-  pstring name;
-  struct in_addr ip;
-  int ttl;
-  unsigned char nb_flags;
-} name_struct;
-
 int num_names=0;
 name_struct *names = NULL;
 
@@ -85,6 +76,10 @@ int add_name(void)
       return(-1);
     }
   n[num_names].valid = False;
+  strcpy(n[num_names].name,"");
+  strcpy(n[num_names].flags,"");
+  n[num_names].ttl = 0;
+  n[num_names].nb_flags = 0;
   names = n;
   num_names++;
   return(num_names-1);
@@ -110,132 +105,132 @@ int find_name(char *s)
   return -1;
 }
 
+
 /****************************************************************************
-true if two IP addresses are equal
+check names, and change any 0 IPs to myip
 ****************************************************************************/
-BOOL ip_equal(struct in_addr *ip1,struct in_addr *ip2)
+void check_names(void)
 {
-  char *p1=(char *)ip1;
-  char *p2=(char *)ip2;
-  int l = sizeof(*ip1);
-  while (l--)
-    if (*p1++ != *p2++)
-      return(False);
-  return(True);
+  int i;
+  for (i=0;i<num_names;i++)
+    if (names[i].valid && strequal((char *)inet_ntoa(names[i].ip),"0.0.0.0"))
+      names[i].ip = myip;
 }
 
+
 /****************************************************************************
-register a name on the net
+dump a copy of the name table
 ****************************************************************************/
-BOOL register_name(char *inbuf,char *outbuf,char *name,struct in_addr *ip)
+void dump_names(void)
 {
-  unsigned char nb_flags = 0;
-  int count;
-  static uint16 name_trn_id = 0x4242;
-  char *p;
+  int i;
+  Debug(3,"Dump of local name table\n");
+  for (i=0;i<num_names;i++)
+    if (names[i].valid)
+      Debug(3,"%s %s %s %d 0x%X\n",
+	    names[i].name,inet_ntoa(names[i].ip),names[i].flags,
+	    names[i].ttl,names[i].nb_flags);
+}
 
-  Debug(1,"Registering name %s (%s) nb_flags=0x%x\n",
-	name, inet_ntoa(*ip) ,nb_flags);
 
-  SSVAL(outbuf,0,name_trn_id++);
-  CVAL(outbuf,2) = (0x5<<3) | 0x1;
-  CVAL(outbuf,3) = (1<<4) | 0x0;
-  SSVAL(outbuf,4,1);
-  SSVAL(outbuf,6,0);
-  SSVAL(outbuf,8,0);
-  SSVAL(outbuf,10,1);  
-  p = outbuf+12;
-  name_mangle(name,p);
-  p += name_len(p);
-  SSVAL(p,0,0x20);
-  SSVAL(p,2,0x1);
-  p += 4;
-  CVAL(p,0) = 0xC0;
-  CVAL(p,1) = 12;
-  p += 2;
-  SSVAL(p,0,0x20);
-  SSVAL(p,2,0x1);
-  SIVAL(p,4,myttl);
-  SSVAL(p,8,6);
-  CVAL(p,10) = nb_flags;
-  CVAL(p,11) = 0;
-  p += 12;
-  memcpy(p,ip,4);
-  p += 4;
-
-  count = 3;
-  while (count--)
+/****************************************************************************
+load a netbios hosts file
+****************************************************************************/
+void load_hosts_file(char *name)
+{
+  FILE *f = fopen(name,"r");
+  pstring line;
+  if (!f) 
     {
-      Debug(2,"Sending reg request for %s at (%s)\n",name,inet_ntoa(*ip));
-
-
-      show_nmb(outbuf);
-      if (!send_packet(outbuf,nmb_len(outbuf),&bcast_ip,137,SOCK_DGRAM))
-	return False;
-      if (receive_nmb(inbuf,3))
-	{
-          int rec_name_trn_id = SVAL(inbuf,0);
-	  int opcode = CVAL(inbuf,2) >> 3;
-	  int nm_flags = ((CVAL(inbuf,2) & 0x7) << 4) + (CVAL(inbuf,3)>>4);
-	  int rcode = CVAL(inbuf,3) & 0xF;
-
-	  /* is it a negative response to our request? */
-	  if ((rec_name_trn_id = name_trn_id) && 
-              (opcode == 5 && nm_flags == 0x58 && rcode != 0 && rcode != 7))
-	    {
-	      char qname[100];
-	      name_extract(inbuf,12,qname);
-	      if (name_equal(qname,name))
-		{
-		  Debug(0,"Someone (%s) gave us a negative name regregistration response!\n",
-			inet_ntoa(lastip));
-		  return False;
-		}
-	    }	  
-	  
-	  /* it's something else - process it anyway, unless we are running
-	   as a daemon. This is necessary as we may have been started by a 
-	   name query of our name arriving on port 137 (often happens) */
-	  if (!daemon)
-	    {
-	      show_nmb(inbuf);
-	      construct_reply(inbuf,outbuf + nmb_len(outbuf));
-	    }
-	}
+      Debug(0,"Couldn't open hosts file %s\n",name);
+      return;
     }
 
-  /* no negative replies, send a demand */
-  p = outbuf;
-  SSVAL(outbuf,0,name_trn_id++);
-  CVAL(outbuf,2) = (0x5<<3);
-  CVAL(outbuf,3) = (1<<4) | 0x0;
-  SSVAL(outbuf,4,1);
-  SSVAL(outbuf,6,0);
-  SSVAL(outbuf,8,0);
-  SSVAL(outbuf,10,1);  
-  p = outbuf+12;
-  name_mangle(name,p);
-  p += name_len(p);
-  SSVAL(p,0,0x20);
-  SSVAL(p,2,0x1);
-  p += 4;
-  CVAL(p,0) = 0xC0;
-  CVAL(p,1) = 12;
-  p += 2;
-  SSVAL(p,0,0x20);
-  SSVAL(p,2,0x1);
-  SIVAL(p,4,myttl);
-  SSVAL(p,8,6);
-  CVAL(p,10) = nb_flags;
-  CVAL(p,11) = 0;
-  p += 12;
-  memcpy(p,ip,4);
-  p += 4;
-  
-  Debug(2,"Sending reg demand for %s at (%s)\n",name,inet_ntoa(*ip));
+  while (!feof(f))
+    {
+      int i;
 
-  show_nmb(outbuf);
-  return(send_packet(outbuf,nmb_len(outbuf),&bcast_ip,137,SOCK_DGRAM));
+      if (!fgets(line,sizeof(pstring),f)) continue;
+      
+      if (*line == '#') continue;
+
+      {
+	pstring ip="",name="",flags="";
+	unsigned long a;
+	int count = sscanf(line,"%s%s%s",ip,name,flags);
+	if (count == 0) continue;
+
+	if (count < 2)
+	  {
+	    Debug(0,"Ill formed hosts line [%s]\n",line);	    
+	    continue;
+	  }
+
+	i = add_name();
+	if (i < 0) 
+	  {
+	    fclose(f);
+	    return;
+	  }
+
+	if (!strchr("0123456789",*ip))
+	  {
+	    struct hostent *hp;
+	    if ((hp = gethostbyname(ip)) == 0) 
+	      {
+		Debug(0,"Couldn't make an IP from [%s]\n",ip);
+		continue;
+	      }
+	    memcpy(&names[i].ip, hp->h_addr, sizeof(names[i].ip));      
+	  }
+	else
+	  {
+	    a = inet_addr(ip);
+	    memcpy(&names[i].ip,&a,sizeof(a));
+	  }
+
+	strupper(name);
+	strcpy(names[i].name,name);
+	strcpy(names[i].flags,flags);
+	if (strchr(flags,'G'))
+	  names[i].nb_flags |= 0x80;
+
+	names[i].valid = True;
+      }      
+    }
+  fclose(f);
+}
+
+
+/****************************************************************************
+add a netbios group name
+****************************************************************************/
+void add_group_name(char *name)
+{
+  int i = add_name();
+  if (i < 0) 
+    return;
+
+  strupper(name);
+  memset(&names[i].ip,0,sizeof(names[i].ip));
+
+  strcpy(names[i].name,name);
+  strcpy(names[i].flags,"G");
+  names[i].nb_flags |= 0x80;
+
+  names[i].valid = True;
+}
+
+
+/****************************************************************************
+register all group names
+****************************************************************************/
+void register_groups(void)
+{
+  int i;
+  for (i=0;i<num_names;i++)
+    if (names[i].valid && (names[i].nb_flags & 0x80))
+      register_name(&names[i],&bcast_ip,daemon?NULL:construct_reply);
 }
 
 
@@ -244,7 +239,7 @@ reply to a reg request
 ****************************************************************************/
 void reply_reg_request(char *inbuf,char *outbuf)
 {
-  int name_trn_id = SVAL(inbuf,0);
+  int rec_name_trn_id = SVAL(inbuf,0);
   char qname[100]="";
   int ttl;
   char *p = inbuf;
@@ -269,20 +264,37 @@ void reply_reg_request(char *inbuf,char *outbuf)
 
   /* if the name doesn't exist yet then don't respond */
   if ((n = find_name(qname)) < 0)
-    return;
+    {
+      Debug(3,"Name doesn't exist\n");
+      return;
+    }
 
   /* if it's a group name and being registered as a group then it's OK */
   if ((names[n].nb_flags & 0x80) && (nb_flags & 0x80))
-    return;
+    {
+      Debug(3,"Group re-registration\n");
+      return;
+    }
 
   /* if it's not my name then don't worry about it */
   if (!name_equal(myname,qname))
-    return;
+    {
+      Debug(3,"Not my name\n");
+      return;
+    }
+
+  /* if it's my name and it's also my IP then don't worry about it */
+  if (ip_equal(&ip,&myip))
+    {
+      Debug(3,"Is my IP\n");
+      return;
+    }
+      
 
   Debug(0,"Someones using my name (%s), sending negative reply\n",qname);
 
   /* Send a NEGATIVE REGISTRATION RESPONSE to protect our name */
-  SSVAL(outbuf,0,name_trn_id);
+  SSVAL(outbuf,0,rec_name_trn_id);
   CVAL(outbuf,2) = (1<<7) | (0x5<<3) | 0x5;
   CVAL(outbuf,3) = (1<<7) | 0x6;
   SSVAL(outbuf,4,0);
@@ -320,67 +332,70 @@ reply to a name query
 ****************************************************************************/
 void reply_name_query(char *inbuf,char *outbuf)
 {
-  int name_trn_id = SVAL(inbuf,0);
+  int rec_name_trn_id = SVAL(inbuf,0);
   char qname[100]="";
   char *p = inbuf;
   unsigned char nb_flags = 0;
   struct in_addr tmpip;
   struct in_addr retip;
+  int i;
 
   name_extract(inbuf,12,qname);
 
   Debug(2,"(%s) querying name (%s)",inet_ntoa(lastip), qname);
 
-  /* if it's not my name then return */
-  if (!name_equal(qname,myname) && !dns_serve)
+  if ((i = find_name(qname)) >= 0)
     {
-      Debug(2,"\n");
-      return;
-    }
-
-  if (name_equal(qname,myname))
-    {
-      memcpy(&retip,&myip,sizeof(retip));
+      retip = names[i].ip;
+      nb_flags = names[i].nb_flags;
       Debug(2," sending positive reply\n");
     }
   else
-    /* try a DNS query to get the IP */
     {
-      struct hostent *hp;
-      pstring hname;
-
-      strcpy(hname,qname);
-      trim_string(hname," "," ");
-
-      if ((hp = gethostbyname(hname)) == 0) 
+      if (!dns_serve)
 	{
-	  Debug(2,": unknown name sending no reply\n");
+	  Debug(2,"\n");
 	  return;
 	}
-
-      memcpy(&retip, hp->h_addr, sizeof(retip));      
-
-      /* If it is on the same subnet then don't send a reply as it
-	 might confuse a client to receive a reply from two hosts. */
-      {
-	unsigned int net1,net2,nmask;
-
-	nmask = *(unsigned int *)&Netmask;
-	net1 = (*(unsigned int *)&myip) & nmask;
-	net2 = (*(unsigned int *)&retip) & nmask;
-
-	if (net1 == net2)
+      else
+	/* try a DNS query to get the IP */
+	{
+	  struct hostent *hp;
+	  pstring hname;
+	  
+	  strcpy(hname,qname);
+	  trim_string(hname," "," ");
+	  
+	  if ((hp = gethostbyname(hname)) == 0) 
+	    {
+	      Debug(2,": unknown name sending no reply\n");
+	      return;
+	    }
+	  
+	  memcpy(&retip, hp->h_addr, sizeof(retip));      
+	  
+	  /* If it is on the same subnet then don't send a reply as it
+	     might confuse a client to receive a reply from two hosts. */
 	  {
-	    Debug(2," on same subnet (%s), sending no reply\n",inet_ntoa(retip));
-	    return;
+	    unsigned int net1,net2,nmask;
+	    
+	    nmask = *(unsigned int *)&Netmask;
+	    net1 = (*(unsigned int *)&myip) & nmask;
+	    net2 = (*(unsigned int *)&retip) & nmask;
+	    
+	    if (net1 == net2)
+	      {
+		Debug(2," on same subnet (%s), sending no reply\n",inet_ntoa(retip));
+		return;
+	      }
 	  }
-      }
+	}
 
       Debug(2," sending positive reply (%s)\n",inet_ntoa(retip));
     }
   
   /* Send a POSITIVE NAME QUERY RESPONSE */
-  SSVAL(outbuf,0,name_trn_id);
+  SSVAL(outbuf,0,rec_name_trn_id);
   CVAL(outbuf,2) = (1<<7) | 0x5;
   CVAL(outbuf,3) = 0;
   SSVAL(outbuf,4,0);
@@ -437,11 +452,17 @@ void process(char *lookup)
   OutBuffer = (char *)malloc(BUFFER_SIZE);
   if ((InBuffer == NULL) || (OutBuffer == NULL)) 
     return;
+
+  register_groups();
   
-  if (!reply_only && !register_name(InBuffer,OutBuffer,myname,&myip))
+  if (!reply_only)
     {
-      Debug(0,"Failed to register my own name\n");
-      return;
+      int i = find_name(myname);
+      if (i < 0 || !register_name(&names[i],&bcast_ip,daemon?NULL:construct_reply))
+	{
+	  Debug(0,"Failed to register my own name\n");
+	  return;
+	}
     }
   
   if (*lookup)
@@ -524,34 +545,9 @@ BOOL open_sockets(BOOL daemon,int port)
 ****************************************************************************/
 BOOL init_structs(void )
 {
-  int i = add_name();  
-  struct hostent *hp;
-
-  if (i < 0)
+  if (!get_myname(myhostname,&myip))
     return(False);
 
-  strcpy(names[i].name,domain);
-  memset(&names[i].ip,0,4);
-  names[i].ttl = 0;
-  names[i].nb_flags = 0x80;
-  names[i].valid = True;
-  
-  /* get my host name */
-  if (gethostname(myhostname, sizeof(myhostname)) == -1) 
-    {
-      Debug(0,"gethostname failed\n");
-      return False;
-    } 
-
-  /* get host info */
-  if ((hp = gethostbyname(myhostname)) == 0) 
-    {
-      Debug(0, "Gethostbyname: Unknown host %s.\n",myhostname);
-      return False;
-    }
-
-  memcpy(&myip,hp->h_addr,4);
-
   /* Read the broadcast address from the interface */
   {
     struct in_addr ip1,ip2;
@@ -565,7 +561,27 @@ BOOL init_structs(void )
   }
 
   if (*myname == 0)
-    strcpy(myname,myhostname);
+    {
+      strcpy(myname,myhostname);
+      strupper(myname);
+    }
+
+  if (find_name(myname) < 0)
+    {
+      int i = add_name();  
+
+      if (i < 0)
+	return(False);
+
+      strcpy(names[i].name,myname);
+      names[i].ip = myip;
+      names[i].ttl = 0;
+      names[i].nb_flags = 0;
+      names[i].valid = True;  
+    }
+  else
+    Debug(3,"Name %s already exists\n",myname);
+
   return True;
 }
 
@@ -586,6 +602,8 @@ void usage(char *pname)
   printf("\t-N netmask           the netmask to use for subnet determination\n");
   printf("\t-L name              lookup this netbios name then exit\n");
   printf("\t-S                   serve queries via DNS if not on the same subnet\n");
+  printf("\t-H hosts file        load a netbios hosts file\n");
+  printf("\t-G group name        add a group name to be part of\n");
   printf("\n");
 }
 
@@ -602,9 +620,15 @@ int main(int argc,char *argv[])
   extern char *optarg;
   pstring lookup = "";
 
-  while ((opt = getopt (argc, argv, "L:B:N:Rn:l:d:Dp:hPS")) != EOF)
+  while ((opt = getopt (argc, argv, "L:B:N:Rn:l:d:Dp:hPSH:G:")) != EOF)
     switch (opt)
       {
+      case 'G':
+	add_group_name(optarg);
+	break;
+      case 'H':
+	load_hosts_file(optarg);
+	break;
       case 'B':
 	{
 	  unsigned long a = inet_addr(optarg);
@@ -683,7 +707,9 @@ int main(int argc,char *argv[])
       Debug(1,"Copyright Andrew Tridgell 1992,1993,1994\n");
     }
   
-  (void)init_structs();
+  init_structs();
+  check_names();
+  dump_names();
 
   if (daemon)
     {
