@@ -55,6 +55,7 @@ typedef unsigned int uint32;
 #define DEFAULT_PIPE_TIMEOUT 10000000 /* Ten seconds */
 #endif
 
+
 /* these macros make access to the packet buffers easier. See also the
 functions SSVAL() and SIVAL(). */
 #define SIVAL(buf,pos,val) sival((char *)(buf),pos,val)
@@ -75,23 +76,7 @@ functions SSVAL() and SIVAL(). */
 /* these define all the command types recognised by the server - there
 are lots of gaps so probably there are some rare commands that are not
 implemented */
-#define pHELLO 114
-#define pCONNECT 112
-#define pFSTAT 16
-#define pDIR 0x81
-#define pDFREE 0x80
-#define pOPEN 2
-#define pCREATE 3
-#define pCLOSE 4
-#define pREAD 10
-#define pWRITE 11
-#define pMKDIR 0
-#define pRMDIR 1
-#define pDELETE 6
-#define pGOODBYE 113
-#define pACCESS 8
-#define pCHMOD 9
-#define pRENAME 7
+
 #define pSETDIR '\377'
 
 /* these define the attribute byte as seen by DOS */
@@ -121,10 +106,89 @@ implemented */
 #define eNOT_SAME_DEVICE	17
 #define eNO_MORE_FILES		18
 
-#define ERRbadfile 2
-#define ERRbadfid 6
-#define ERRbadpath 3
-#define ERRbadpw   2
+/* SMB X/Open error codes for the ERRdos error class */
+
+#define ERRbadfunc 1 /* Invalid function (or system call) */
+#define ERRbadfile 2 /* File not found (pathname error) */
+#define ERRbadpath 3 /* Directory not found */
+#define ERRnofids 4 /* Too many open files */
+#define ERRnoaccess 5 /* Access denied */
+#define ERRbadfid 6 /* Invalid fid */
+#define ERRnomem 8 /* Out of memory */
+#define ERRbadmem 9 /* Invalid memory block address */
+#define ERRbadenv 10 /* Invalid environment */
+#define ERRbadaccess 12 /* Invalid open mode */
+#define ERRbaddata 13 /* Invalid data (only from ioctl call) */
+#define ERRres 14 /* reserved */
+#define ERRbaddrive 15 /* Invalid drive */
+#define ERRremcd 16 /* Attempt to delete current directory */
+#define ERRdiffdevice 17 /* rename/move across different filesystems */
+#define ERRnofiles 18 /* no more files found in file search */
+#define ERRbadshare 32 /* Share mode on file conflict with open mode */
+#define ERRlock 33 /* Lock request conflicts with existing lock */
+#define ERRfilexists 80 /* File in operation already exists */
+#define ERRbadpipe 230 /* Named pipe invalid */
+#define ERRpipebusy 231 /* All instances of pipe are busy */
+#define ERRpipeclosing 232 /* named pipe close in progress */
+#define ERRnotconnected 233 /* No process on other end of named pipe */
+#define ERRmoredata 234 /* More data to be returned */
+#define ERROR_EAS_DIDNT_FIT 275 /* Extended attributes didn't fit */
+#define ERROR_EAS_NOT_SUPPORTED 282 /* Extended attributes not suppored */
+
+/* Error codes for the ERRSRV class */
+
+#define ERRerror 1 /* Non specific error code */
+#define ERRbadpw 2 /* Bad password */
+#define ERRbadtype 3 /* reserved */
+#define ERRaccess 4 /* No permissions to do the requested operation */
+#define ERRinvnid 5 /* tid invalid */
+#define ERRinvnetname 6 /* Invalid servername */
+#define ERRinvdevice 7 /* Invalid device */
+#define ERRqfull 49 /* Print queue full */
+#define ERRqtoobig 50 /* Queued item too big */
+#define ERRinvpfid 52 /* Invalid print file in smb_fid */
+#define ERRsmbcmd 64 /* Unrecognised command */
+#define ERRsrverror 65 /* smb server internal error */
+#define ERRfilespecs 67 /* fid and pathname invalid combination */
+#define ERRbadlink 68 /* reserved */
+#define ERRbadpermits 69 /* Access specified for a file is not valid */
+#define ERRbadpid 70 /* reserved */
+#define ERRsetattrmode 71 /* attribute mode invalid */
+#define ERRpaused 81 /* Message server paused */
+#define ERRmsgoff 82 /* Not receiving messages */
+#define ERRnoroom 83 /* No room for message */
+#define ERRrmuns 87 /* too many remote usernames */
+#define ERRtimeout 88 /* operation timed out */
+#define ERRnoresource  89 /* No resources currently available for request. */
+#define ERRtoomanyuids 90 /* too many userids */
+#define ERRbaduid 91 /* bad userid */
+#define ERRuseMPX 250 /* temporarily unable to use raw mode, use MPX mode */
+#define ERRuseSTD 251 /* temporarily unable to use raw mode, use standard mode */
+#define ERRcontMPX 252 /* resume MPX mode */
+#define ERRbadPW /* reserved */
+#define ERRnosupport 0xFFFF
+
+/* Error codes for the ERRHRD class */
+
+#define ERRnowrite 19 /* read only media */
+#define ERRbadunit 20 /* Unknown device */
+#define ERRnotready 21 /* Drive not ready */
+#define ERRbadcmd 22 /* Unknown command */
+#define ERRdata 23 /* Data (CRC) error */
+#define ERRbadreq 24 /* Bad request structure length */
+#define ERRseek 25
+#define ERRbadmedia 26
+#define ERRbadsector 27
+#define ERRnopaper 28
+#define ERRwrite 29 /* write fault */
+#define ERRread 30 /* read fault */
+#define ERRgeneral 31 /* General hardware failure */
+#define ERRwrongdisk 34
+#define ERRFCBunavail 35
+#define ERRsharebufexc 36 /* share buffer exceeded */
+#define ERRdiskfull 39
+
+
 
 typedef char pstring[MAXPATHLEN];
 
@@ -138,6 +202,18 @@ typedef struct
 	pstring name;
 	} files_struct;
 
+/* this is the structure used for the local netbios name table */
+typedef struct
+{
+  BOOL valid;
+  pstring name;
+  char flags[10];
+  struct in_addr ip;
+  int ttl;
+  unsigned char nb_flags;
+} name_struct;
+
+
 typedef struct
 {
   BOOL open;
@@ -241,7 +317,7 @@ typedef struct
 #define SMBexit       0x11   /* process exit */
 #define SMBlseek      0x12   /* seek */
 #define SMBtcon       0x70   /* tree connect */
-#define SMBtconX       0x75   /* tree connect and X*/
+#define SMBtconX      0x75   /* tree connect and X*/
 #define SMBtdis       0x71   /* tree disconnect */
 #define SMBnegprot    0x72   /* negotiate protocol */
 #define SMBdskattr    0x80   /* get disk attributes */
@@ -288,7 +364,7 @@ typedef struct
 #define SMBopenX         0x2D   /* open and X */
 #define SMBreadX         0x2E   /* read and X */
 #define SMBwriteX        0x2F   /* write and X */
-#define SMBsesssetup     0x73   /* Session Set Up & X (including User Logon) */
+#define SMBsesssetupX    0x73   /* Session Set Up & X (including User Logon) */
 #define SMBtconX         0x75   /* tree connect and X */
 #define SMBffirst        0x82   /* find first */
 #define SMBfunique       0x83   /* find unique */
@@ -303,12 +379,6 @@ typedef struct
 #define ERRCMD 0xFF  /* Command was not in the "SMB" format. */
 
 
-#define ERRnosupport 0xFFFF
-#define ERRinvnid 5
-#define ERRaccess 4
-
-#define ERRnoresource  89 /* No resources currently available for request. */
-
 #ifdef USE_DIRECT
 #define DIRECT direct
 #else
@@ -405,6 +475,15 @@ void replacestr(char *str1, char *str2, int start, int n);
 void openstr(char *s, int start, int n);
 void closestr(char *s, int start, int n);
 BOOL set_filetime(char *fname,time_t mtime);
+char *dirname_dos(char *path,char *buf);
+char *filename_dos(char *path,char *buf);
+BOOL get_myname(char *myname,struct in_addr *ip);
+BOOL ip_equal(struct in_addr *ip1,struct in_addr *ip2);
+void expand_mask(char *Mask);
+BOOL sane_unix_date(time_t unixdate);
+time_t start_of_month(void);
+void log_out(char *buffer,int len);
+BOOL register_name(name_struct *name,struct in_addr *destip,void (*)());
 #ifdef NOSTRDUP
 char *strdup(char *s);
 #endif
@@ -521,5 +600,15 @@ typedef struct
 } file_info;
 
 
+
+/* Macros to get at offsets within smb_lkrng and smb_unlkrng
+   structures. We cannot define these as actual structures
+   due to possible differences in structure packing
+   on different machines/compilers. */
+
+#define SMB_LPID(buf,indx) (*(uint16 *)(((char *)(buf)) + (10 * (indx))))
+#define SMB_LKOFF(buf,indx) (*(uint32 *)(((char *)(buf)) + 2 + (10 * (indx))))
+#define SMB_LKLEN(buf,indx) (*(uint32 *)(((char *)(buf)) + 6 + (10 * (indx))))
+
 #endif 
 
