@@ -70,7 +70,7 @@ va_dcl
   if (!dbf) return;
   
 #ifdef __STDC__
-  va_start(ap, level);
+  va_start(ap, format_str);
 #else
   va_start(ap);
   debug_level = va_arg(ap,int);
@@ -791,7 +791,8 @@ return the absolute current directory path
 ********************************************************************/
 char *GetWd(char *s)
 {
-#if (defined(LINUX) || defined(SOLARIS) || defined(SVR4) || defined(HPUX) || defined(NEXT))
+#if (defined(LINUX) || defined(SOLARIS) || defined(SVR4) || \
+     defined(HPUX) || defined(NEXT) || defined(ISC))
   return((char *)getcwd(s,sizeof(pstring)));
 #else
   return((char *)getwd(s));
@@ -919,30 +920,23 @@ BOOL reduce_name(char *s,char *dir)
 
 
 /****************************************************************************
-  see if a name matches a mask. The mask takes the form of several characters,
-  with ? being a wild card.
+expand a wildcard expression, replacing *s with ?s
 ****************************************************************************/
-BOOL mask_match(char *Name,char *Mask,BOOL dodots,BOOL case_sensitive)
+void expand_mask(char *Mask)
 {
-  char *p1,*p2;
-  pstring nbeg=""; /* beginning of name */
-  pstring next=""; /* extension of name */
-  pstring mext=""; /* extension of mask */
-  pstring mbeg=""; /* beg of mask */  
-  pstring name,mask;
+  pstring mbeg,mext;
+  pstring dirpart="";
+  pstring filepart="";
+  char *p1;
+  BOOL absolute = (*Mask == '\\');
 
-  Debug(3,"mmatch [%s] [%s] %d\n",name,mask,dodots);
+  /* parse the directory and filename */
+  if (strchr(Mask,'\\'))
+    dirname_dos(Mask,dirpart);
 
-  strcpy(name,Name);
-  strcpy(mask,Mask);
+  filename_dos(Mask,filepart);
 
-  if (!case_sensitive)
-    {
-      strlower(name);
-      strlower(mask);
-    }
-
-  strcpy(mbeg,mask);
+  strcpy(mbeg,filepart);
   if ((p1 = strchr(mbeg,'.')) != NULL)
     {*p1 = 0;p1++;strcpy(mext,p1);}
   else
@@ -985,18 +979,107 @@ BOOL mask_match(char *Name,char *Mask,BOOL dodots,BOOL case_sensitive)
       strcpy(tmp + l1 + lfill,mext + l1 + 1);	
       strcpy(mext,tmp);
     }
-  
-  strcpy(nbeg,name);
-  if ((p1 = strchr(nbeg,'.')) != NULL)
-    {*p1 = 0;p1++;strcpy(next,p1);}
+
+  strcpy(Mask,dirpart);
+  if (*dirpart || absolute) strcat(Mask,"\\");
+  strcat(Mask,mbeg);
+  strcat(Mask,".");
+  strcat(Mask,mext);
+
+  Debug(6,"Mask expanded to [%s]\n",Mask);
+
+}  
+
+
+/****************************************************************************
+  see if a name matches a mask. The mask takes the form of several characters,
+  with ? being a wild card.
+****************************************************************************/
+BOOL mask_match(char *Name,char *Mask,BOOL dodots,BOOL case_sensitive)
+{
+  char *p1,*p2;
+  pstring nbeg=""; /* beginning of name */
+  pstring next=""; /* extension of name */
+  pstring mext=""; /* extension of mask */
+  pstring mbeg=""; /* beg of mask */  
+  pstring name,mask;
+
+  Debug(3,"mmatch [%s] [%s] %d\n",name,mask,dodots);
+
+  strcpy(name,Name);
+  strcpy(mask,Mask);
+
+  if (!case_sensitive)
+    {
+      strlower(name);
+      strlower(mask);
+    }
+
+  strcpy(mbeg,mask);
+  if ((p1 = strchr(mbeg,'.')) != NULL)
+    {
+      *p1 = 0;
+      p1++;
+      strcpy(mext,p1);
+    }
   else
-    strcpy(next,"");
-  
+    {
+      strcpy(mext,"");
+      if (strlen(mbeg) > 8)
+	{
+	  strcpy(mext,mbeg + 8);
+	  mbeg[8] = 0;
+	}
+    }
+
+  if (*mbeg == 0)
+    strcpy(mbeg,"????????");
+  if (*mext == 0)
+    strcpy(mext,"???");
+
+  /* expand *'s */
+  while ((p1 = strchr(mbeg,'*')) != NULL)
+    {
+      int lfill = 9 - strlen(mbeg);
+      int l1= (p1 - mbeg);
+      int i;
+      pstring tmp="";
+      strcpy(tmp,mbeg);      
+      for (i=0;i<lfill;i++)
+	tmp[l1 + i] = '?';
+      strcpy(tmp + l1 + lfill,mbeg + l1 + 1);	
+      strcpy(mbeg,tmp);      
+    }
+  while ((p1 = strchr(mext,'*')) != NULL)
+    {
+      int lfill = 4 - strlen(mext);
+      int l1= (p1 - mext);
+      int i;
+      pstring tmp="";
+      strcpy(tmp,mext);      
+      for (i=0;i<lfill;i++)
+	tmp[l1 + i] = '?';
+      strcpy(tmp + l1 + lfill,mext + l1 + 1);	
+      strcpy(mext,tmp);
+    }
   
   /* a couple of special cases */
   if (strequal(name,".") || strequal(name,".."))
     return(dodots && strequal(mbeg,"????????") && strequal(mext,"???"));
   
+  strcpy(nbeg,name);
+  if ((p1 = strchr(nbeg,'.')) != NULL)
+    {
+      *p1 = 0;
+      p1++;
+      strcpy(next,p1);
+      if (strchr(next,'.')) /* can't have two .s in a name */
+	return(False);
+    }
+  else
+    strcpy(next,"");
+  
+  
   if (strlen(nbeg) == 0) return(False);
   if (strlen(mbeg) == 0) return(False);
   if (strlen(nbeg) > 8) return(False);
@@ -1134,7 +1217,7 @@ int read_socket(int fd,char *buf,int len)
     {
       lastip = *(struct in_addr *) &sock.sa_data[2];
       if (DEBUGLEVEL > 0)
-	Debug(3,"read %d bytes from (%s)\n",ret,inet_ntoa(lastip));
+	Debug(3,"read %d bytes\n",ret);
     }
   return(ret);
 #endif
@@ -1634,33 +1717,6 @@ BOOL receive_nmb(char *buffer,int timeout)
 }
 
 
-/****************************************************************************
-send a name message
-****************************************************************************/
-BOOL send_nmb(int fd,char *buffer)
-{
-  int len;
-  int ret,nwritten=0;
-  len = nmb_len(buffer);
-
-  while (nwritten < len)
-    {
-      ret = write_socket(fd,buffer+nwritten,len - nwritten);
-      if (ret <= 0)
-	{
-	  Debug(0,"Error writing %d bytes to client. %d. Exiting\n",len,ret);
-	  return(False);
-	}
-      nwritten += ret;
-    }
-
-  log_out(buffer,len);
-
-  return(True);
-}
-
-
-
 /****************************************************************************
 find a pointer to a netbios name
 ****************************************************************************/
@@ -1709,7 +1765,7 @@ void show_nmb(char *inbuf)
   int arcount = SVAL(inbuf,10);
   char name[100];
 
-  Debug(2,"\nPACKET INTERPRETATION\n");
+  Debug(3,"\nPACKET INTERPRETATION\n");
 
 #if 0
   if (dbf)
@@ -1718,79 +1774,79 @@ void show_nmb(char *inbuf)
 #endif
 
   if (opcode == 5 && ((nm_flags & ~1) == 0x10) && rcode == 0)
-    Debug(2,"NAME REGISTRATION REQUEST (%s)\n",nm_flags&1?"Broadcast":"Unicast");
+    Debug(3,"NAME REGISTRATION REQUEST (%s)\n",nm_flags&1?"Broadcast":"Unicast");
 
   if (opcode == 5 && ((nm_flags & ~1) == 0x00) && rcode == 0)
-    Debug(2,"NAME OVERWRITE REQUEST AND DEMAND (%s)\n",nm_flags&1?"Broadcast":"Unicast");
+    Debug(3,"NAME OVERWRITE REQUEST AND DEMAND (%s)\n",nm_flags&1?"Broadcast":"Unicast");
   
   if (opcode == 9 && ((nm_flags & ~1) == 0x00) && rcode == 0)
-    Debug(2,"NAME REFRESH REQUEST (%s)\n",nm_flags&1?"Broadcast":"Unicast");
+    Debug(3,"NAME REFRESH REQUEST (%s)\n",nm_flags&1?"Broadcast":"Unicast");
   
   if (opcode == 5 && nm_flags == 0x58 && rcode == 0)
-    Debug(2,"POSITIVE NAME REGISTRATION RESPONSE\n");
+    Debug(3,"POSITIVE NAME REGISTRATION RESPONSE\n");
   
   if (opcode == 5 && nm_flags == 0x58 && rcode != 0 && rcode != 7)
-    Debug(2,"NEGATIVE NAME REGISTRATION RESPONSE\n");
+    Debug(3,"NEGATIVE NAME REGISTRATION RESPONSE\n");
   
   if (opcode == 5 && nm_flags == 0x50 && rcode == 0)
-    Debug(2,"END-NODE CHALLENGE REGISTRATION RESPONSE\n");
+    Debug(3,"END-NODE CHALLENGE REGISTRATION RESPONSE\n");
   
   if (opcode == 5 && nm_flags == 0x58 && rcode != 0 && rcode == 7)
-    Debug(2,"NAME CONFLICT DEMAND\n");
+    Debug(3,"NAME CONFLICT DEMAND\n");
   
   if (opcode == 6 && (nm_flags&~1) == 0x00 && rcode == 0)
-    Debug(2,"NAME RELEASE REQUEST & DEMAND (%s)\n",nm_flags&1?"Broadcast":"Unicast");
+    Debug(3,"NAME RELEASE REQUEST & DEMAND (%s)\n",nm_flags&1?"Broadcast":"Unicast");
   
   if (opcode == 6 && (nm_flags&~1) == 0x40 && rcode == 0)
-    Debug(2,"POSITIVE NAME RELEASE RESPONSE\n");
+    Debug(3,"POSITIVE NAME RELEASE RESPONSE\n");
   
   if (opcode == 6 && (nm_flags&~1) == 0x40 && rcode != 0)
-    Debug(2,"NEGATIVE NAME RELEASE RESPONSE\n");
+    Debug(3,"NEGATIVE NAME RELEASE RESPONSE\n");
   
   if (opcode == 0 && (nm_flags&~1) == 0x10 && rcode == 0)
-    Debug(2,"NAME QUERY REQUEST (%s)\n",nm_flags&1?"Broadcast":"Unicast");
+    Debug(3,"NAME QUERY REQUEST (%s)\n",nm_flags&1?"Broadcast":"Unicast");
   
   if (opcode == 0 && (nm_flags&~0x28) == 0x50 && rcode == 0)
-    Debug(2,"POSITIVE NAME QUERY RESPONSE\n");
+    Debug(3,"POSITIVE NAME QUERY RESPONSE\n");
   
   if (opcode == 0 && (nm_flags&~0x08) == 0x50 && rcode != 0)
-    Debug(2,"NEGATIVE NAME QUERY RESPONSE\n");
+    Debug(3,"NEGATIVE NAME QUERY RESPONSE\n");
   
   if (opcode == 0 && nm_flags == 0x10 && rcode == 0)
-    Debug(2,"REDIRECT NAME QUERY RESPONSE\n");
+    Debug(3,"REDIRECT NAME QUERY RESPONSE\n");
 
   if (opcode == 7 && nm_flags == 0x80 && rcode == 0)
-    Debug(2,"WAIT FOR ACKNOWLEDGEMENT RESPONSE\n");
+    Debug(3,"WAIT FOR ACKNOWLEDGEMENT RESPONSE\n");
   
   if (opcode == 0 && (nm_flags&~1) == 0x00 && rcode == 0)
-    Debug(2,"NODE STATUS REQUEST (%s)\n",nm_flags&1?"Broadcast":"Unicast");
+    Debug(3,"NODE STATUS REQUEST (%s)\n",nm_flags&1?"Broadcast":"Unicast");
 
   if (opcode == 0 && nm_flags == 0x80 && rcode == 0)
-    Debug(2,"NODE STATUS RESPONSE\n");
+    Debug(3,"NODE STATUS RESPONSE\n");
   
   
-  Debug(2,"name_trn_id=0x%x\nopcode=0x%x\nnm_flags=0x%x\nrcode=0x%x\n",
+  Debug(3,"name_trn_id=0x%x\nopcode=0x%x\nnm_flags=0x%x\nrcode=0x%x\n",
 	name_trn_id,opcode,nm_flags,rcode);
-  Debug(2,"qdcount=%d\nancount=%d\nnscount=%d\narcount=%d\n",
+  Debug(3,"qdcount=%d\nancount=%d\nnscount=%d\narcount=%d\n",
 	qdcount,ancount,nscount,arcount);
 
   l = 12;
   for (i=0;i<qdcount;i++)
     {
       int type,class;
-      Debug(2,"QUESTION %d\n",i);
+      Debug(3,"QUESTION %d\n",i);
       name_extract(inbuf,l,name);
       l += name_len(inbuf+l);
       type = SVAL(inbuf+l,0);
       class = SVAL(inbuf+l,2);
       l += 4;
-      Debug(2,"\t%s\n\ttype=0x%x\n\tclass=0x%x\n",name,type,class);
+      Debug(3,"\t%s\n\ttype=0x%x\n\tclass=0x%x\n",name,type,class);
     }
 
   for (i=0;i<(ancount + nscount + arcount);i++)
     {
       int type,class,ttl,rdlength;
-      Debug(2,"RESOURCE %d\n",i);
+      Debug(3,"RESOURCE %d\n",i);
       name_extract(inbuf,l,name);
       l += name_len(inbuf + l);
       type = SVAL(inbuf+l,0);
@@ -1798,11 +1854,11 @@ void show_nmb(char *inbuf)
       ttl = IVAL(inbuf+l,4);
       rdlength = SVAL(inbuf+l,8);
       l += 10 + rdlength;
-      Debug(2,"\t%s\n\ttype=0x%x\n\tclass=0x%x\n",name,type,class);
-      Debug(2,"\tttl=%d\n\trdlength=%d\n",ttl,rdlength);
+      Debug(3,"\t%s\n\ttype=0x%x\n\tclass=0x%x\n",name,type,class);
+      Debug(3,"\tttl=%d\n\trdlength=%d\n",ttl,rdlength);
     }
 
-  Debug(2,"\n");
+  Debug(3,"\n");
   
 }
 
@@ -1827,8 +1883,6 @@ BOOL send_packet(char *buf,int len,struct in_addr *ip,int port,int type)
   struct sockaddr_in sock_out;
   int one=1;
 
-  log_out(buf,len);
-
   if (passive)
     return(True);
 
@@ -1853,7 +1907,6 @@ BOOL send_packet(char *buf,int len,struct in_addr *ip,int port,int type)
   /* log the packet */
   log_out(buf,len);
 
-
   if (DEBUGLEVEL > 0)
     Debug(3,"sending a packet of len %d to (%s) on port %d of type %s\n",
 	  len,inet_ntoa(*ip),port,type==SOCK_DGRAM?"DGRAM":"STREAM");
@@ -1977,7 +2030,7 @@ void become_daemon(void)
   setpgrp();
 #endif
 
-#ifdef HPUX
+#if (defined(HPUX) || defined(ISC))
   setsid();
 #else
   i = open("/dev/tty", O_RDWR);
@@ -2003,7 +2056,7 @@ BOOL get_broadcast(struct in_addr *if_ipaddr, struct in_addr *if_bcast, struct i
   int i;
   
   /* Create a socket to the INET kernel. */
-#ifdef sun386
+#if (defined(sun386) || defined(ULTRIX))
   if ((sock = socket(AF_INET, SOCK_DGRAM, 0 )) < 0)
 #else
   if ((sock = socket(AF_INET, SOCK_RAW, PF_INET )) < 0)
@@ -2423,6 +2476,22 @@ char *dirname_dos(char *path,char *buf)
 }
 
 
+/****************************************************************************
+parse out a filename from a path name. Assumes dos style filenames.
+****************************************************************************/
+char *filename_dos(char *path,char *buf)
+{
+  char *p = strrchr(path,'\\');
+
+  if (!p)
+    strcpy(buf,path);
+  else
+    strcpy(buf,p+1);
+
+  return(buf);
+}
+
+
 struct
 {
   int thresh;
@@ -2562,3 +2631,285 @@ int Strlen(char *s)
   return(ret);
 }
 #endif
+
+
+/****************************************************************************
+return a time at the start of the current month
+****************************************************************************/
+time_t start_of_month(void)
+{
+  time_t t = time(NULL);
+  struct tm *t2;
+  
+  t2 = gmtime(&t);
+  
+  t2->tm_mday = 1;
+  t2->tm_hour = 0;
+  t2->tm_min = 0;
+  t2->tm_sec = 0;
+  
+  return(mktime(t2) - TimeDiff());
+}
+
+
+/*******************************************************************
+  check for a sane unix date
+********************************************************************/
+BOOL sane_unix_date(time_t unixdate)
+{
+  struct tm t,today;
+  time_t t_today = time(NULL);
+  
+  t = *(LocalTime(&unixdate));
+  today = *(LocalTime(&t_today));
+  
+  if (t.tm_year < 80)
+    return(False);
+  
+  if (t.tm_year >  today.tm_year)
+    return(False);
+  
+  if (t.tm_year == today.tm_year &&
+      t.tm_mon > today.tm_mon)
+    return(False);
+  
+  
+  if (t.tm_year == today.tm_year &&
+      t.tm_mon == today.tm_mon &&
+      t.tm_mday > (today.tm_mday+1))
+    return(False);
+  
+  return(True);
+}
+
+
+
+#ifdef ISC
+ /*******************************************************************
+ftruncate for ISC 
+********************************************************************/
+int ftruncate(int f,long l)
+{
+      struct  flock   fl;
+
+      fl.l_whence = 0;
+      fl.l_len = 0;
+      fl.l_start = l;
+      fl.l_type = F_WRLCK;
+      return fcntl(f, F_FREESP, &fl);
+}
+#endif
+
+
+/****************************************************************************
+register a netbios name on the net.
+****************************************************************************/
+BOOL register_name(name_struct *name,struct in_addr *destip,void (*fn)())
+{
+  int count;
+  char *p;
+  char *inbuf = (char *)malloc(BUFFER_SIZE);
+  char *outbuf = (char *)malloc(BUFFER_SIZE);
+  BOOL saved_swap = NeedSwap;
+  static uint16 name_trn_id = 0;
+
+  if (name_trn_id == 0) name_trn_id = getpid() + (time(NULL) % 1000);
+
+  NeedSwap = !big_endian();
+
+  if ((inbuf == NULL) || (outbuf == NULL)) 
+    {
+      NeedSwap = saved_swap;
+      return False;
+    }
+
+  name_trn_id++;
+
+  Debug(1,"Registering name %s (%s) nb_flags=0x%x\n",
+	name->name, inet_ntoa(name->ip) ,name->nb_flags);
+
+  SSVAL(outbuf,0,name_trn_id);
+  CVAL(outbuf,2) = (0x5<<3) | 0x1;
+  CVAL(outbuf,3) = (1<<4) | 0x0;
+  SSVAL(outbuf,4,1);
+  SSVAL(outbuf,6,0);
+  SSVAL(outbuf,8,0);
+  SSVAL(outbuf,10,1);  
+  p = outbuf+12;
+  name_mangle(name->name,p);
+  p += name_len(p);
+  SSVAL(p,0,0x20);
+  SSVAL(p,2,0x1);
+  p += 4;
+  CVAL(p,0) = 0xC0;
+  CVAL(p,1) = 12;
+  p += 2;
+  SSVAL(p,0,0x20);
+  SSVAL(p,2,0x1);
+  SIVAL(p,4,0); /* my own ttl */
+  SSVAL(p,8,6);
+  CVAL(p,10) = name->nb_flags;
+  CVAL(p,11) = 0;
+  p += 12;
+  memcpy(p,&(name->ip),4);
+  p += 4;
+
+  count = 3;
+  while (count--)
+    {
+      Debug(3,"Sending reg request for %s at (%s)\n",
+	    name->name,inet_ntoa(name->ip));
+
+
+      show_nmb(outbuf);
+      if (!send_packet(outbuf,nmb_len(outbuf),destip,137,SOCK_DGRAM))
+	{
+	  free(inbuf);free(outbuf);
+	  NeedSwap = saved_swap;
+	  return False;
+	}
+
+      if (receive_nmb(inbuf,3))
+	{
+          int rec_name_trn_id = SVAL(inbuf,0);
+	  int opcode = CVAL(inbuf,2) >> 3;
+	  int nm_flags = ((CVAL(inbuf,2) & 0x7) << 4) + (CVAL(inbuf,3)>>4);
+	  int rcode = CVAL(inbuf,3) & 0xF;
+
+	  /* is it a negative response to our request? */
+	  if ((rec_name_trn_id == name_trn_id) && 
+              (opcode == 5 && nm_flags == 0x58 && rcode != 0 && rcode != 7))
+	    {
+	      char qname[100];
+	      name_extract(inbuf,12,qname);
+	      if (name_equal(qname,name->name))
+		{
+		  Debug(0,"Someone (%s) gave us a negative name regregistration response!\n",
+			inet_ntoa(lastip));
+		  free(inbuf);free(outbuf);
+		  NeedSwap = saved_swap;
+		  return False;
+		}
+	      else
+		{
+		  Debug(0,"%s gave negative name regregistration for %s??\n",
+			inet_ntoa(lastip),qname);
+		}		  
+	    }	  
+	  
+	  /* it's something else - process it anyway, unless we are running
+	   as a daemon. This is necessary as we may have been started by a 
+	   name query of our name arriving on port 137 (often happens) */
+	  if (fn)
+	    {
+	      show_nmb(inbuf);
+	      fn(inbuf,outbuf + nmb_len(outbuf));
+	    }
+	}
+    }
+
+  /* increment the packet id */
+  name_trn_id++;
+
+  /* don't demand on groups */
+  if ((name->nb_flags & 0x80) != 0)
+    {
+      free(inbuf);free(outbuf);
+      NeedSwap = saved_swap;
+      return(True);
+    }
+
+  /* no negative replies, send a demand */
+  p = outbuf;
+  SSVAL(outbuf,0,name_trn_id);
+  CVAL(outbuf,2) = (0x5<<3);
+  CVAL(outbuf,3) = (1<<4) | 0x0;
+  SSVAL(outbuf,4,1);
+  SSVAL(outbuf,6,0);
+  SSVAL(outbuf,8,0);
+  SSVAL(outbuf,10,1);  
+  p = outbuf+12;
+  name_mangle(name->name,p);
+  p += name_len(p);
+  SSVAL(p,0,0x20);
+  SSVAL(p,2,0x1);
+  p += 4;
+  CVAL(p,0) = 0xC0;
+  CVAL(p,1) = 12;
+  p += 2;
+  SSVAL(p,0,0x20);
+  SSVAL(p,2,0x1);
+  SIVAL(p,4,0); /* my own ttl */
+  SSVAL(p,8,6);
+  CVAL(p,10) = name->nb_flags;
+  CVAL(p,11) = 0;
+  p += 12;
+  memcpy(p,&(name->ip),4);
+  p += 4;
+  
+  Debug(3,"Sending reg demand for %s at (%s)\n",
+	name->name,inet_ntoa(name->ip));
+
+  show_nmb(outbuf);
+
+  {
+    BOOL ret = send_packet(outbuf,nmb_len(outbuf),destip,137,SOCK_DGRAM);
+
+    free(inbuf);free(outbuf);
+  
+    NeedSwap = saved_swap;
+    return(ret);
+  }
+}
+
+
+/****************************************************************************
+get my own name and IP
+****************************************************************************/
+BOOL get_myname(char *myname,struct in_addr *ip)
+{
+  struct hostent *hp;
+  pstring myhostname;
+
+  /* get my host name */
+  if (gethostname(myhostname, sizeof(myhostname)) == -1) 
+    {
+      Debug(0,"gethostname failed\n");
+      return False;
+    } 
+  
+  /* get host info */
+  if ((hp = gethostbyname(myhostname)) == 0) 
+    {
+      Debug(0, "Gethostbyname: Unknown host %s.\n",myhostname);
+      return False;
+    }
+
+  if (myname)
+    {
+      strcpy(myname,myhostname);
+      strupper(myname);
+    }
+
+  if (ip)
+    memcpy(ip,hp->h_addr,4);
+
+  return(True);
+}
+
+
+/****************************************************************************
+true if two IP addresses are equal
+****************************************************************************/
+BOOL ip_equal(struct in_addr *ip1,struct in_addr *ip2)
+{
+  char *p1=(char *)ip1;
+  char *p2=(char *)ip2;
+  int l = sizeof(*ip1);
+  while (l--)
+    if (*p1++ != *p2++)
+      return(False);
+  return(True);
+}
+
+
